@@ -36,7 +36,7 @@
 
 #include "udisksdaemon.h"
 #include "udisksdaemonutil.h"
-#include "udisksstate.h"
+#include "udiskscleanup.h"
 #include "udiskslogging.h"
 #include "udiskslinuxblockobject.h"
 #include "udiskslinuxdriveobject.h"
@@ -44,8 +44,6 @@
 #if defined(HAVE_LIBSYSTEMD_LOGIN)
 #include <systemd/sd-daemon.h>
 #include <systemd/sd-login.h>
-
-#define LOGIND_AVAILABLE() (access("/run/systemd/seats/", F_OK) >= 0)
 #endif
 
 /**
@@ -260,13 +258,10 @@ udisks_daemon_util_resolve_link (const gchar *path,
   if (num != -1)
     {
       char *absolute_path;
-      gchar *full_path_dir;
 
       link_path[num] = '\0';
 
-      full_path_dir = g_path_get_dirname (full_path);
-      absolute_path = g_build_filename (full_path_dir, link_path, NULL);
-      g_free (full_path_dir);
+      absolute_path = g_build_filename (path, link_path, NULL);
       if (realpath (absolute_path, resolved_path) != NULL)
         {
           found_it = TRUE;
@@ -343,20 +338,20 @@ udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
   gboolean ret;
   UDisksBlock *block = NULL;
   UDisksPartition *partition = NULL;
-  UDisksState *state;
+  UDisksCleanup *cleanup;
   uid_t setup_by_user;
   UDisksObject *crypto_object;
 
   ret = FALSE;
 
-  state = udisks_daemon_get_state (daemon);
+  cleanup = udisks_daemon_get_cleanup (daemon);
   block = udisks_object_get_block (object);
   if (block == NULL)
     goto out;
   partition = udisks_object_get_partition (object);
 
   /* loop devices */
-  if (udisks_state_has_loop (state, udisks_block_get_device (block), &setup_by_user))
+  if (udisks_cleanup_has_loop (cleanup, udisks_block_get_device (block), &setup_by_user))
     {
       if (setup_by_user == user)
         {
@@ -388,9 +383,9 @@ udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
     {
       UDisksBlock *crypto_block;
       crypto_block = udisks_object_peek_block (crypto_object);
-      if (udisks_state_find_unlocked_luks (state,
-                                           udisks_block_get_device_number (crypto_block),
-                                           &setup_by_user))
+      if (udisks_cleanup_find_unlocked_luks (cleanup,
+                                             udisks_block_get_device_number (crypto_block),
+                                             &setup_by_user))
         {
           if (setup_by_user == user)
             {
@@ -402,20 +397,6 @@ udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
       g_object_unref (crypto_object);
     }
 
-  /* MDRaid devices */
-  if (g_strcmp0 (udisks_block_get_mdraid (block), "/") != 0)
-    {
-      uid_t started_by_user;
-      if (udisks_state_has_mdraid (state, udisks_block_get_device_number (block), &started_by_user))
-        {
-          if (started_by_user == user)
-            {
-              ret = TRUE;
-              goto out;
-            }
-        }
-    }
-
  out:
   g_clear_object (&partition);
   g_clear_object (&block);
@@ -637,31 +618,7 @@ udisks_daemon_util_check_authorization_sync (UDisksDaemon          *daemon,
       _safe_polkit_details_insert (details, "drive.model", udisks_drive_get_model (drive));
       _safe_polkit_details_insert (details, "drive.revision", udisks_drive_get_revision (drive));
       if (udisks_drive_get_removable (drive))
-        {
-          const gchar *const *media_compat;
-          GString *media_compat_str;
-          const gchar *sep = ",";
-
-          polkit_details_insert (details, "drive.removable", "true");
-          _safe_polkit_details_insert (details, "drive.removable.bus", udisks_drive_get_connection_bus (drive));
-
-          media_compat_str = g_string_new (NULL);
-          media_compat = udisks_drive_get_media_compatibility (drive);
-          if (media_compat)
-            {
-              guint i;
-
-              for (i = 0; media_compat[i] && strlen(media_compat[i]); i++)
-                {
-                  if (i)
-                    g_string_append (media_compat_str, sep);
-                  g_string_append (media_compat_str, media_compat[i]);
-                }
-            }
-
-          _safe_polkit_details_insert (details, "drive.removable.media", media_compat_str->str);
-          g_string_free (media_compat_str, TRUE);
-        }
+        polkit_details_insert (details, "drive.removable", "true");
     }
 
   if (block != NULL)
@@ -1065,8 +1022,8 @@ udisks_daemon_util_on_same_seat (UDisksDaemon          *daemon,
   UDisksObject *drive_object = NULL;
   UDisksDrive *drive = NULL;
 
-  /* if we don't have logind, assume it's always the same seat */
-  if (!LOGIND_AVAILABLE())
+  /* if we haven't booted with systemd, assume it's always the same seat */
+  if (sd_booted () <= 0)
     return TRUE;
 
   if (UDISKS_IS_LINUX_BLOCK_OBJECT (object))
@@ -1424,64 +1381,3 @@ udisks_daemon_util_uninhibit_system_sync (UDisksInhibitCookie *cookie)
 #endif
 }
 
-/**
- * udisks_daemon_util_get_free_mdraid_device:
- *
- * Gets a free MD RAID device.
- *
- * Returns: A string of the form "/dev/mdNNN" that should be freed
- * with g_free() or %NULL if no free device is available.
- */
-gchar *
-udisks_daemon_util_get_free_mdraid_device (void)
-{
-  gchar *ret = NULL;
-  gint n;
-  gchar buf[PATH_MAX];
-
-  /* Ideally we wouldn't need this racy function... but mdadm(8)
-   * insists that the user chooses a name. It should just choose one
-   * itself but that's not how things work right now.
-   */
-  for (n = 127; n >= 0; n--)
-    {
-      snprintf (buf, sizeof buf, "/sys/block/md%d", n);
-      if (!g_file_test (buf, G_FILE_TEST_EXISTS))
-        {
-          ret = g_strdup_printf ("/dev/md%d", n);
-          goto out;
-        }
-    }
-
- out:
-  return ret;
-}
-
-
-/**
- * udisks_ata_identify_get_word:
- * @identify_data: (allow-none): A 512-byte array containing ATA IDENTIFY or ATA IDENTIFY PACKET DEVICE data or %NULL.
- * @word_number: The word number to get - must be less than 256.
- *
- * Gets a <quote>word</quote> from position @word_number from
- * @identify_data.
- *
- * Returns: The word at the specified position or 0 if @identify_data is %NULL.
- */
-guint16
-udisks_ata_identify_get_word (const guchar *identify_data, guint word_number)
-{
-  const guint16 *words = (const guint16 *) identify_data;
-  guint16 ret = 0;
-
-  g_return_val_if_fail (word_number < 256, 0);
-
-  if (identify_data == NULL)
-    goto out;
-
-  ret = GUINT16_FROM_LE (words[word_number]);
-
- out:
-  return ret;
-}
-
