@@ -35,7 +35,6 @@
 #include "udisksdaemon.h"
 #include "udiskscleanup.h"
 #include "udisksdaemonutil.h"
-#include "udiskslinuxdevice.h"
 
 /**
  * SECTION:udiskslinuxpartitiontable
@@ -111,11 +110,11 @@ udisks_linux_partition_table_update (UDisksLinuxPartitionTable  *table,
                                      UDisksLinuxBlockObject     *object)
 {
   const gchar *type = NULL;
-  UDisksLinuxDevice *device = NULL;;
+  GUdevDevice *device = NULL;;
 
   device = udisks_linux_block_object_get_device (object);
   if (device != NULL)
-    type = g_udev_device_get_property (device->udev_device, "ID_PART_TABLE_TYPE");
+    type = g_udev_device_get_property (device, "ID_PART_TABLE_TYPE");
 
   udisks_partition_table_set_type_ (UDISKS_PARTITION_TABLE (table), type);
 
@@ -347,16 +346,13 @@ handle_create_partition (UDisksPartitionTable   *table,
    * will be replaced by the name of the drive/device in question
    */
   message = N_("Authentication is required to create a partition on $(drive)");
-  if (!udisks_daemon_util_setup_by_user (daemon, object, caller_uid))
+  if (udisks_block_get_hint_system (block))
     {
-      if (udisks_block_get_hint_system (block))
-        {
-          action_id = "org.freedesktop.udisks2.modify-device-system";
-        }
-      else if (!udisks_daemon_util_on_same_seat (daemon, object, caller_pid))
-        {
-          action_id = "org.freedesktop.udisks2.modify-device-other-seat";
-        }
+      action_id = "org.freedesktop.udisks2.modify-device-system";
+    }
+  else if (!udisks_daemon_util_on_same_seat (daemon, object, caller_pid))
+    {
+      action_id = "org.freedesktop.udisks2.modify-device-system-other-seat";
     }
 
   if (!udisks_daemon_util_check_authorization_sync (daemon,
